#------------------------------------------------------------------
#
# Sept. 2007 - Christine Yan (cyan@cisco.com)
# Jan. 2009(Updated) - Vimaladevi Rajakanthan (vrajakan@cisco.com)
# 
#
# Copyright (c) 2007-2009 by cisco Systems, Inc.
# All rights reserved.
#
# Version-date: v1.0-09/09/2007
#
# Name: lib.tcl
#
# Desc: lib procedures to be shared by eem policies
#
#------------------------------------------------------------------

global msg_delimiter \
    "--------------------------------------------------------------"

###############################################################################
# 
#   Description: IOS super user command.
#                This is for getting ios super command under enable mode. 
#                such as show command etc.
#
#   IN: ios cli command which needs to be executed. 
#   OUT: None.
#
###############################################################################

proc ios_enable_command { ios_command } {

    if [catch {cli_open} result] {
        error $result "Cannot open file channel for enable command"
    } else {
	array set cli1 $result
    }

    # From the following cli, we can get fd for show command.
    if [catch {cli_exec $cli1(fd) "enable"} result] {
        error $result "Cannot go to enable mode"
    }

    if [catch {cli_exec $cli1(fd) "term len 0"} result] {
        error $result "Cannot configure terminal mode"
    }

    if [catch {cli_exec $cli1(fd) $ios_command } result] {
        error $result "Cannot exec ios command"
    } else {
	set cmd_out $result
    }

    if [catch {cli_close $cli1(fd) $cli1(tty_id)} result] {
        error $result "Cannot close exec channel"
    }

    return $cmd_out
  
}   

proc ping_command {pingAddress} {

    global verbose
    
    if [catch {cli_open} result] {
        error $result "Cannot open router session"
    } else {
        array set cli $result
    }

    if [catch {cli_exec $cli(fd) "ping $pingAddress"} result] {
        error $result "Cannot execute ping"
    } else {
        set cmd_output $result
    }
    # close cli
    if [catch {cli_close $cli(fd) $cli(tty_id)} result] {
        error $result "Cannot close router session"
    }

    # Ping output looks something like this:
    #  Type escape sequence to abort.
    #  Sending 5, 100-byte ICMP Echos to 193.72.216.2, timeout is 2 seconds:
    #  !!!!!
    #  Success rate is 100 percent (5/5), round-trip min/avg/max = 1/1/4 ms
    #
    # We want the number after "Success rate is "

    # We set a default in case the ping command fails and returns
    # something unusual. Let's just assume this like a 0% ping success 
    set percentage "0"  

    # Now let's get the success rate
    regexp {Success rate is ([0-9]*) percent} $cmd_output dummy percentage
    set successRateArray $percentage

    if { $verbose == 1 } {
        puts "success rate is  $successRateArray "
    }
    return $successRateArray
 
}

##############################################################################
#
#  Description:   send_no_history_syslog_msg
#                 Send standard no history notification out, so all scripts 
#                 would be consistent.       
#
#  IN:            None
#  OUT:           None
#
##############################################################################

proc send_no_history_syslog_msg {} {

    set msg_delimiter "----------------------------------------------------"
    append msg_delimiter "----------"

    action_syslog msg "\nHistory info will not be sent when email is\
	    disabled\n$msg_delimiter"

}

##############################################################################
#
#  Description:   get_history_msg
#                 Get configuration change from router
#
#  IN: None      
#  OUT: The historic configuration change
#
##############################################################################

proc get_histroy_msg {} {

    set msg_header "\n\nTHE CONFIGURATION HISTORY is: "
    set msg $msg_header
    set h_msg [ios_enable_command "sh archive config diff"]
    append msg $h_msg
    if { $msg == $msg_header } {
	append msg \n\n "none"
    }

    append msg \n\n "----------------------------------------------------"
    append msg "----------"
    return $msg

}

##############################################################################
#
#  Description:   send_no_event_history_syglog_msg
#                 Send standard no event history notification out, so all 
#                 scripts would be consistent.       
#
#  IN: None      
#  OUT: None
#
##############################################################################

proc send_no_event_history_syslog_msg {} {

    set msg_delimiter "----------------------------------------------------"
    append msg_delimiter "----------"

    action_syslog msg "\nEvent History info will not be sent when email is\
	    disabled \n$msg_delimiter"

}


##############################################################################
#
#  Description:   get_event_history_msg
#                 Get configuration change from router
#
#  IN: None      
#  OUT: The historic configuration change
#
##############################################################################

proc get_event_histroy_msg {} {

    set msg_header "\n\nTHE EVENT HISTORY is: "
    set msg $msg_header
    set h_msg [ios_enable_command "sh logging"]
    append msg $h_msg
    if { $msg == $msg_header } {
	append msg \n\n "none"
    }

    append msg \n\n "----------------------------------------------------"
    append msg "----------"
    return $msg

}

##############################################################################
#
#  Description:   send_no_email_syslog_msg
#                 Send standard no email notification out, so all scripts 
#                 would be consistent.       
#
#  IN: N/A
#      
#  OUT: N/A
#
##############################################################################

proc send_no_email_syslog_msg {} {

    set msg_delimiter "----------------------------------------------------"
    append msg_delimiter "----------"

    action_syslog msg "\nEmail result will not be sent, please look at\
	    console or buffer on service diagnostic messages\n$msg_delimiter"
}

##############################################################################
#
#  Description: check_if_required_ios_configured            
#
#  IN: required configuation command 
#      
#  OUT: If found, return 1
#       If not found, return 0
#
##############################################################################

proc check_if_required_ios_configured { required_config_command } {

    set ios_cli "show run | include $required_config_command"
    set cli_out [ios_enable_command $ios_cli]
    
    if { [regexp "$required_config_command" $cli_out dummy] } {
	return 1
    } else {
	return 0
    }
}


##############################################################################
#
#  Description:        send_info
#
#  IN: config_history  if config history is needed
#      event_history   if event history is needed
#
#  OUT: None
#
##############################################################################

proc send_info { config_history event_history } {
    
    if { $config_history == 1 } {
        send_no_history_syslog_msg
    }

    if { $event_history == 1 } {
        send_no_event_history_syslog_msg
    }

    send_no_email_syslog_msg
}

##############################################################################
#
#  Description:        send_email_result
#
#  IN: _email_from   environment variable for sending email
#      _email_to     environment variable for sending email
#      _email_cc     environment variable for sending email
#      _email_server environment variable for sending email
#
#  OUT: None
#
##############################################################################

proc send_email_result { msg _email_from _email_to _email_cc _email_server} {

    global cmd_output
    global routername

    # If $_email_server is an ip address, there is no need to have ip domain
    # is configured, otherwise it is needed.

    set required_domain_name_config \
	    [check_if_required_ios_configured "ip domain"]
    if { $required_domain_name_config == 0 } {
        action_syslog msg "\nYou have to config \"ip domain (name)\"\
                first in order to send email. Please note, some Cisco\
                IOS versions need to config \"ip domain-name\""
        return
    } else {
        if ![regexp {([0-9]+).([0-9]+).([0-9]+).([0-9]+)} $_email_server] {
	    set required_nameserver_config \
		    [check_if_required_ios_configured "ip name-server"]
	    if { $required_nameserver_config == 0 } {
		action_syslog msg "\nYou have to config \"ip name-server\"\
			first"
		return
	    }
	}
    }

    set routername [info hostname]

    if {[string match "" $routername]} {
	error "Host name is not configured"
    }

    set show_eem_user_lib "show event man dir user library"
    set cli_out [ios_enable_command $show_eem_user_lib]
    set usr_lib [lindex $cli_out 0]

    if { $usr_lib == "" } {
        puts "Cannot send email without template"
	return
    }

    set cmd_output $msg

    if [catch {smtp_subst [file join $usr_lib email_template_cmd]} \
	    result] {
	error $result "Wrong email template cmd file"
    }

    if [catch {smtp_send_email $result} result] {
	error $result "Email can not be sent out"
    }

    set msg_delimiter "----------------------------------------------------"
    append msg_delimiter "----------"

    action_syslog msg "\nEmail has been sent to $_email_to, please check\
		your email box for diagnostic details\n$msg_delimiter"
}

##############################################################################
#
#  Description:        send_result_by_email
#
#  IN: config_history  if config history is needed
#      event_history   if event history is needed
#      msg             diag messages from policy script
#      _email_from   environment variable for sending email
#      _email_to     environment variable for sending email
#      _email_cc     environment variable for sending email
#      _email_server environment variable for sending email
#
#  OUT: None
#
##############################################################################

proc send_result_by_email { config_history event_history msg _email_from \
     _email_to _email_cc _email_server } {

    global msg_delimiter

    if {$_email_from == ""} {
        action_syslog msg "Email can not be sent because no valid _email_from\
            value\$msg_delimiter"
        return
    }

    if {$_email_to == ""} {
        action_syslog msg "Email can not be sent because no valid _email_from\
            value\$msg_delimiter"
        return
    }

    if { $config_history == 1 } {
        set history_msg [get_histroy_msg]
        append msg $history_msg
    }

    if { $event_history == 1 } {
        set event_history_msg [get_event_histroy_msg]
        append msg $event_history_msg
    }

    send_email_result $msg $_email_from $_email_to $_email_cc $_email_server

}

###############################################################################
#
#   Description: Executes the show run command and pipes the output to the
#                section filter with the keywords for the filter passed as
#                input and returns the configuration for the section specified
#                in the input.
#
#   IN: keywords that specify which section to be parsed
#   OUT: A section of show run output
#
###############################################################################

proc show_run_section {keywords} {

    set section_output ""
    set section_end FALSE
    set output [ios_enable_command "show run | begin $keywords"]
    set result [split $output "\n"]
    for {set a 0} {$a < [llength $result]} {incr a} {
        set y [expr $a+1]
        set line [lindex $result $y]
        if {[string index $line 0] == " "} {
            set config [lindex $result $a]
            append section_output "$config\n"
        } else {
            set section_end TRUE
            set config [lindex $result $a]
            append section_output "$config"
        }

        if {$section_end == "TRUE"} {
            return $section_output
        }

    }
}

###############################################################################
#
#   Description: Given the first octet of network ip address the procedure
#                checks for the range of the octet and decides whether it
#                belongs to "class A" or "class "B or "class C" network and
#                returns the  corresponding subnet mask.
#
#   IN : First octet of a network ip address
#   OUT: The subnet mask to which the network ip address belongs
#
###############################################################################

proc find_class_subnet {network} {

    set subnet ""
    set ip [split $network "."]
    set number [lindex $ip 0]
    if {[string is integer $number] == 1} {
       if {($number >= 0) && ($number <= 127)} {
            set subnet "255.0.0.0"
        } elseif {($number >= 128) && ($number <= 191)} {
            set subnet "255.255.0.0"
        } elseif {($number >= 192) && ($number <= 223)} {
            set subnet "255.255.255.0"
        } else {
            # do nothing
        }
    } else {
        errorMessage "lib.tcl" $number "WHEN EXECUTING 'find_class_subnet'"
    }

    return $subnet
}

