::cisco::eem::event_register_timer watchdog name bgp_neighbor_formation sec $_svcdiag_bgpFormTimer maxrun 300

#------------------------------------------------------------------------------
# EEM policy to check "BGP neighbor formation problems" which includes ibgp and 
#                      ebgp neighbors
#
# June 2007 - Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007-2009 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-06/08/2007 
# Title: Report all the problems related to BGP neighbor formation to customer
# Name: bgp_neighbor_formation.tcl
# Desc: This script troubleshoots the causes for unsuccessful neighbor
#       formation like "missing update-source configuration" "mis-configured 
#       access-list" for ibgp neighbor
# 
#       In addition to the above problems it also troubleshoots "missing
#       ebgp-multihop configuration" for ebgp neighbors
#
#       The structure and requirment of the file is defined below.
#
#       For the purpose of test automation an environment variable
#       _automation is required. This environment variable is checked if
#       defined previously and if not is defined.
# 
# Input-file:
#
#
# Environment Variables for this script are:
# _svcdiag_bgpFormTimer
# _svcdiag_bgpFormConfigHistory
# _svcdiag_bgpFormEventHistory
# _svcdiag_bgpFormNotif
# _automation
# 
#------------------------------------------------------------------------------

# check if all the environment variables related to email exist
# If any of them doesn't exist, print out an error message and quit

if {($_svcdiag_bgpFormNotif == "email") || ($_svcdiag_bgpFormNotif == "all")} {
    if {![info exists _email_server]} {
        set result \
            "Policy cannot be run: variable _email_server has not been set"
        error $result "_email_server is required for sending email"
    }

    if {![info exists _email_from]} {
        set result \
            "Policy cannot be run: variable _email_from has not been set"
        error $result "_email_from is required for sending email"
    }

    if {![info exists _email_to]} {
        set result \
            "Policy cannot be run: variable _email_to has not been set"
        error $result "_email_to is required for sending email"
    }

    if {![info exists _email_cc]} {
        #_email_cc is an option, must set to empty string if not set.
        set _email_cc ""
    }
}

if {![info exists _automation]} {
    set _automation FALSE
}

# The namespace import commands that follow import some cisco TCL extensions
# that are required to run this script

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

# Initialize a variable to store the messages that need to be sent to the user

global send_email email policy 
set policy "bgp_neighbor_formation.tcl"
set messageString "\nTHE SERVICE DIAGNOSTIC MESSAGE FOR BGP NEIGHBOR FORMATION \
                   IS:\n" 
set send_email 0

###############################################################################
#    Description: Determines whether BGP connection is ibgp or ebgp 
#    
#    IN: Local AS number and Remote AS number
#    OUT: Returns ibgp or ebgp
#
###############################################################################

proc findBGPType {value1 value2}  {

    if {$value1 == $value2} {
        set type "ibgp"
    } else {
        set type "ebgp" 
    }
    return $type
} 

###############################################################################
#   
#    Description: Determines whether BGP connection is in directly connected
#                 subnet or indirectly connected subnet 
#    
#    IN: IP address of the router 
#    OUT: Returns "direct" if in directly connected subnet or "Null" if there
#         is no output and "indirect" otherwise 
#         indirectly connected subnet 
# 
###############################################################################

proc findIfDirectlyConnected {ipaddr} {

    global policy 
    set result ""
    set ip_route_output [exec_show_commands "show ip route $ipaddr"]

    if {([regexp {% Network not in table} $ip_route_output a] == 1) || 
        ([regexp {% Subnet not in table} $ip_route_output b] == 1) } {
        set result "Null"
    } elseif {[regexp {Known via \"connected\"} $ip_route_output b] == 1} {
        set result "direct"
    } elseif {[regexp {Invalid input detected at} $ip_route_output] == 1 } {
        errorMessage $policy $ipaddr "WHEN EXECUTING 'show ip route $ipaddr'"
    } else {
        set result "indirect"
    }
    return $result
}

################################################################################
#
#    Description: Determines if the neighbor is reachable via unicast ping
#
#    IN: IP address of the neighbor
#    OUT: Returns "1" if ping is successful or "0" for unsuccessful ping  
#
################################################################################

proc exec_ping {ipaddr} {
   
    global policy 
    set val 0
    set ping_output [exec_show_commands "ping $ipaddr"]
    if {[regexp {[0-9]+ percent} $ping_output a] == 1} {
        set percent [split $a " "]
    } else {
        errorMessage $policy $ipaddr "WHEN EXECUTING ping $ipaddr"    
        exit 0
    }

    if {[lindex $percent 0] > 50 } {
        set val 1   
    } else {
        set val 0
    }
    return $val
}

################################################################################
#
#    Description: Determines if "neighbor x.x.x.x update-source Loopback#" is
#                 configured on the router
#
#    IN: show run output, ip address of the neighbor and peer-group name
#    OUT: Returns error message string about missing configuration or an empty
#         string 
#
################################################################################

proc checkUpdateSource {sh_run ipaddr pgname} {
   
    set config ""
    set msgString ""
    
    foreach line [split $sh_run "\n"] {
        if {[regexp "neighbor $ipaddr update-source Loopback" $line a] == 1} {
            regsub -all {[ \t]+} $line " " out       
            set out [string trim $out]
            set ip [split $out " "]
            if {[lindex $ip 1] == $ipaddr} { 
                set config "configured" 
                break 
            }
        } else {
            if {[regexp "neighbor $pgname update-source" $line b] == 1} {
                set config "configured" 
                break
            }
        }
    }

    if {$config == ""} {
        append msgString "The configuration 'neighbor $ipaddr update-source\
        Loopback'  may be required on the local router.\n"
    }
    
    return $msgString 
}

################################################################################
#
#    Description: Determines if "neighbor x.x.x.x ebgp-multihop#" is
#                 configured on the router
#
#    IN: show run output, ip address of the neighbor and peer-group name
#    OUT: Returns error message string about missing configuration or an empty
#         string 
#
################################################################################

proc checkEbgpMultihop {sh_run ipaddr pgname} {

    set config ""
    set msgString "" 
    
    foreach line [split $sh_run "\n"] {
        if {[regexp "neighbor $ipaddr ebgp-multihop" $line a] == 1} {
            regsub -all {[ \t]+} $line " " out       
            set out [string trim $out]
            set ip [split $out " "]
            if {[lindex $ip 1] == $ipaddr} { 
                set config "configured" 
                break 
            }
        } else {
            if {[regexp "neighbor $pgname ebgp-multihop" $line b] == 1} {
                set config "configured"
                break
            } 
        }
    }

    if {$config == ""} {
        append msgString "The configuration 'neighbor $ipaddr ebgp-multihop'\
        may be required on the local router.\n"
    }
    return $msgString
}

###############################################################################
#
#    Description: Finds out the interface which has the BGP connection 
#                 established with the same neighbor
#                 
#    IN : IP addresss of the BGP neighbor
#    OUT : Returns the IP address or the name of the interface through which
#          BGP connection is established   
#
###############################################################################

proc findBgpConfigIntf {ipaddr} {
   
    set intf "" 
    set ip_route_output [exec_show_commands "show ip route $ipaddr"]
    if {([regexp {% Subnet not in table} $ip_route_output] == "0") || \
        ([regexp {% Network not in table} $ip_route_output] == "0")} {  
        set a [split $ip_route_output "\n"]
        for {set i 0} {$i < [llength $a]} {incr i} {
            if {[regexp {Routing Descriptor Blocks:} [lindex $a $i]] == 1} {
                set b [lindex $a [expr $i+1]]
                set c [split $b " "]
                for {set j 0} {$j < [llength $c]} {incr j} {
                    if {[regexp {via} [lindex $c $j]] == 1} {    
                        set intf [lindex $c [expr $j+1]]
                    }
                }
            }
        }
    }
    return $intf
    
}

################################################################################
#
#    Description: Determines if access list is configured on the interface
#                 that is used for BGP Connection
#
#    IN: IP address of the neighbor 
#    OUT: Returns message string "configured" if access list is configured and
#         "not configured" if the access list is not configured on the
#         interface that is used for BGP connection 
#
################################################################################

proc checkAccessListConfig {ipaddr} {

    set msg "not configured"
    set bgp_intf ""
    set intf [findBgpConfigIntf $ipaddr]
    if {[regexp { [0-9]+\.[0-9]+\.[0-9]+\.[0-9]+ } $intf] == 1} {
        set bgp_intf [findBgpConfigIntf $intf] 
    }

    if {$bgp_intf != ""} {
        set intf $bgp_intf
    }

    set output [exec_show_commands "show run interface $intf"]
    if {$output != ""} {
        foreach line [split $output "\n"] {
            if {[regexp { ip access-group [0-9]*} $line a] == 1} {
                set msg "configured"    
            }
        } 
    }
    
    return $msg
}

################################################################################
#
#    Description: Executes extended ping for ibgp neighbors connected through 
#                  loopback address
#
#    IN: show run output  and the ip address of the neighbor
#    OUT: Returns message string "fail" if ping fails and "pass" if ping 
#         passes  
#
################################################################################

proc extendedPing {ipaddr sh_run} {

    set ping_output [exec_show_commands "ping $ipaddr"]

    if {[regexp "!!!!" $ping_output] == 0 } {
        set msgString "fail"
    } else {
        set msgString "pass"
    }

    return $msgString
}

################################################################################
#
#    Description: Determines if the routing table on the local router has a
#                 route to the loopback interface of the peer router and 
#                 returns appropriate message
#
#    IN: ip address of the neighbor
#    OUT: Returns appropriate message string for missing route to the loopback
#         interface of the local router or peer router 
#
################################################################################

proc checkRoutingTable {ipaddr} {

    set msgString ""
    set output [exec_show_commands "show ip route $ipaddr"]
    if {[regexp {% Subnet not in table} $output] == "1"} {
        set msgString "Neighbor $ipaddr does not seem to be reachable\
        through ping. Check network connectivity and routing between peers.\n"  
    } else {
       append msgString "Check BGP configuration and routing at the peer and\
       any Access list restrictions between the peers.\n"
    }
    return $msgString
}

################################################################################
#
#    Description: Finds out the number of neighbors and iterates for each
#                 neighbor to find if the neighbor is an ibgp neighbor or ebgp
#                 neighbor.  
#
#    IN: show run output, local AS number, remote AS number and the ip address
#        of the neighbor and peer-group name
#    OUT: Returns error message string about missing configuration for
#         update-source, ebgp-multihop or mis-configured access-list or an
#         error message that the reason is not known 
#
################################################################################

proc troubleshootIbgpEbgpProblem {locAS remoAS ipaddr sh_run pg_name} {

    global policy
    set msgString ""

#### different AS number ###  
       
        set bgp_type [findBGPType $locAS $remoAS] 
        set bgpConnectionType "" 

        if {$bgp_type == "ibgp"} {
            set bgpConnectionType IBGP
        } else {
            set bgpConnectionType EBGP 
        }
       
        switch -exact -- $bgpConnectionType {
             IBGP {

### Check for IBGP ###
                set type [findIfDirectlyConnected $ipaddr]
                 
                if {$type == "direct"} {
### Directly connected ibgp ###
                    if {[exec_ping $ipaddr] == "1"} {
                        if {[checkAccessListConfig $ipaddr] == "configured"} {
                            append msgString "Neighbor $ipaddr is reachable\
                            through ping. Check BGP configuration at the peer\
                            and/or any access list restrictions between the\
                            peers.\n"
                        } else {
                            append msgString "Neighbor $ipaddr is reachable\
                            through ping. Check BGP configuration at the peer\
                            and/or any access list restrictions between the\
                            peers.\n"
                        }   
                    } else {
                        append msgString "Neighbor $ipaddr does not seem to\
                        be reachable through ping. Check network\
                        connectivity, BGP configuration at the peer and/or any\
                        access list restrictions between the peers.\n"
                    } 
                } elseif {$type == "indirect"} {
                
### Indirectly connected ibgp ###
                    set message1 "Check BGP configuration and BGP neighbor\
                                  reachability  for neighbor $ipaddr at the peer and/or any\
                                  access-list restrictions between the peers.\n"
                    set message2 [checkUpdateSource $sh_run $ipaddr $pg_name] 
                    
                    if {[exec_ping $ipaddr] == "0"} {
                        append msgString $message1
                        append msgString $message2
                    } else {
                        if {$message2 == ""} {
                            append msgString "Check BGP configuration and BGP neighbor\
                                              reachability at the peer for neighbor $ipaddr and\
                                              any access-list restrictions between the peers.\n"
                        } else {
                            append msgString $message2
                        }
                    }    

                } elseif {$type == "Null"} {
                    append msgString "\nCheck network connectivity to the BGP neighbor $ipaddr\n" 
                } else {
                    append msgString "The reason for BGP neighbor formation problem for \
                                       neighbor $ipaddr cannot be diagnosed at this time"
                }
             }
                        
            EBGP {

### Check for EBGP ###
                
                set subnetType [findIfDirectlyConnected $ipaddr] 
                if {$subnetType == "direct"} {
                    if {[exec_ping $ipaddr] == "1"} {
                        if {[checkAccessListConfig $ipaddr] == "configured"} {
                            append msgString "Neighbor $ipaddr is reachable\
                            through ping. Check BGP configuration at the peer\
                            and/or any access list restrictions between the\
                            peers.\n" 
                        } else {
                            append msgString "Neighbor $ipaddr is reachable\
                            through ping. Check BGP configuration at the peer\
                            and/or any access list restrictions between the\
                            peers.\n" 
                        }
                    } else {
                        append msgString "Neighbor $ipaddr does not seem to\
                        be reachable through ping. Check network connectivity,\
                        BGP configuration at the peer and/or any access list\
                        restrictions between the peers.\n" 
                    }
                } elseif {$subnetType == "indirect"} {
                    set message1 "Check BGP configuration and BGP neighbor reachability at the\
                                  peer for neighbor $ipaddr and/or any access-list\
                                  restrictions between the peers.\n"
                    set message2 [checkUpdateSource $sh_run $ipaddr $pg_name] 
                    set message3 [checkEbgpMultihop $sh_run $ipaddr $pg_name] 
                    
                    if {[exec_ping $ipaddr] == "0"} {
                        append msgString $message1
                        append msgString "$message2 $message3"
                    } else {
                        if {($message2 == "") && ($message3 == "")} {
                            append msgString $message1
                        } else {
                            append msgString "$message2 $message3"
                        }
                    }    
                    
                } elseif {$subnetType == "Null"} {
                    append msgString "Check network connectivity to the BGP neighbor $ipaddr"
                } else {
                    append msgString "The reason for BGP neighbor formation problem for \
                                      neighbor $ipaddr cannot be diagnosed at this time"
                }
          
            }
                           
            default {
            }
        } 
        
### end of switch ###
        
    return $msgString
    
} 

### Policy execution starts from here ###

### get IP address of neighbor and remote-as ###

set email "0"
set localAS ""
set state ""
set neighborIPAddr {}
set remoteAS {}
set show_run_output [exec_show_commands "show run"]
set peer_grp_name ""

# parse "show ip bgp summary" output to get the local AS number

set bgp_summary [exec_show_commands "show ip bgp summary"] 
set keywords "Neighbor MsgSent State/PfxRcd" 
set bgp_sum_table [parse_table $bgp_summary $keywords]
set table_values [parse_header_and_values $bgp_sum_table]

if {$bgp_summary != ""} {
    if {[regexp {local AS number [0-9]+} $bgp_summary match] == 1} {
        set temp [split $match " "]
        set localAS [lindex $temp 3] 
    }
}

# Parse "show ip bgp summary" to get a list of the neighbor ip addresses that
# have problems and their assoicated remote AS numbers

set table_value_len [llength $table_values] 
if {$table_value_len != 0} {
    for {set i 0} {$i < $table_value_len} {incr i} {
        set row [split [lindex $table_values $i] " "]
        set state [lindex $row 9]
        if {[regexp {Idle|Connect|Active} $state match] == 1} {
            lappend neighborIPAddr [lindex $row 0]
            lappend remoteAS [lindex $row 2]
        }
    }
}

set ipaddr_list_len [llength $neighborIPAddr]

if {$ipaddr_list_len != 0} {
    for {set i 0} {$i < [llength $neighborIPAddr]} {incr i} { 
        set email "1"
        set ipaddr [lindex $neighborIPAddr $i]
        set peer_grp_name [parse_peer_group_name $ipaddr] 
            append messageString [troubleshootIbgpEbgpProblem $localAS [lindex $remoteAS $i] $ipaddr $show_run_output $peer_grp_name]
    }
}

set configHistory ""
set eventHistory ""

### TO LOG CONFIGURATION HISTORY ###

if {$_svcdiag_bgpFormConfigHistory == "TRUE"} {
    append configHistory \n\n " THE CONFIGURATION HISTORY is: " 
    append configHistory [exec_show_commands "sh archive config diff"]
    append configHistory "-------------------------------------------------------------------------------"
}

### TO LOG EVENT HISTORY ###

if {$_svcdiag_bgpFormEventHistory == "TRUE"} {
    append eventHistory \n\n " THE EVENT HISTORY is : "
    append eventHistory [exec_show_commands "sh logging"] 
    append eventHistory "-------------------------------------------------------------------------------" 
}

if {$email == "1"} {
    if {$_svcdiag_bgpFormNotif == "syslog"} {
        action_syslog msg $messageString
    } elseif {$_svcdiag_bgpFormNotif == "all"} {
        set send_email 1
        action_syslog msg $messageString
        append messageString $configHistory
        append messageString $eventHistory
    } else {
        if {$_svcdiag_bgpFormNotif == "email"} {
            set send_email 1
            append messageString $configHistory
            append messageString $eventHistory
        }
    }
}

set output [exec_show_commands "show event man dir user lib"]
if {[regexp "not configured" $output match]} {
    error "User library directory is not configured"
}
set user_lib_dir [lindex $output 0]

###  send the email out ###

if {$email == "1"} {
    if {$send_email == "1"} {
        set routername [info hostname]
    
        if {[string match "" $routername]} {
            error "Host name is not configured"
        }

        if [catch {smtp_subst [file join $user_lib_dir email_template]} result] {
            puts "There is some error in file join"
            error $result $errorInfo
        }

        if [catch {smtp_send_email $result} result] {
            error $result $errorInfo
        } else {
            if {$_automation == "TRUE"} {
                puts $messageString
            }
        }
    }
}
