::cisco::eem::event_register_syslog occurs 1 pattern "\%BGP-5-ADJCHANGE: neighbor *.*.*.* Down" maxrun 300

#------------------------------------------------------------------------------
# EEM policy to check "BGP neighbor loss problem" due to access-list  
#                      configuration on the router
#
# June 2007 - Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007-2008 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-06/08/2007 
# Title: Report the BGP neighbor loss problem to customer
# Name: bgp-neighbor_loss.tcl
# Desc: This script troubleshoots neighbor loss problem due to access-list 
#       configuration on the router 
#
#       The structure and requirment of the file is defined below.
#       
#       For the purpose of test automation an environment variable _automation
#       is required. This environment variable is checked if defined
#       previously and if not is defined.
# 
# Input-file:
#
#
# Environment Variables for this script are:
# _svcdiag_bgpLossNotif
# _svcdiag_bgpLossEventHistory
# _svcdiag_bgpLossConfigHistory 
# _automation
# 
#------------------------------------------------------------------------------


# check if all the environment variables we need exist
# If any of them doesn't exist, print out an error message and quit

if {($_svcdiag_bgpLossNotif == "email") || ($_svcdiag_bgpLossNotif == "all")} { 
    if {![info exists _email_server]} {
        set result \
            "Policy cannot be run: variable _email_server has not been set"
        error $result "_email_server is required for sending email" 
    }

    if {![info exists _email_from]} {
        set result \
            "Policy cannot be run: variable _email_from has not been set"
        error $result "_email_from is required for sending email"
    }

    if {![info exists _email_to]} {
        set result \
            "Policy cannot be run: variable _email_to has not been set"
        error $result "_email_to is required for sending email"
    }

    if {![info exists _email_cc]} {
        #_email_cc is an option, must set to empty string if not set.
        set _email_cc ""
    }
}

if {![info exists _automation]} {
    set _automation FALSE
}

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

global messageString 
array set arr_einfo [event_reqinfo]
set messageString "\nTHE SERVICE DIAGNOSTIC MESSAGE FOR BGP NEIGHBOR LOSS IS:\n"
set syslog_error_message $arr_einfo(msg)

set send_email 0

### Get the ip address of the neighbor ###

foreach word [split $syslog_error_message :] {
    set error_message_string $word
}

set ip_address [lindex $error_message_string 1]

###############################################################################
#
#    Description: Finds out the interface which has the BGP connection
#                 established with the same neighbor
#                 
#    IN : IP addresss of the BGP neighbor
#    OUT : Returns the IP address or the name of the interface through which
#          BGP connection is established
#   
###############################################################################

proc findBgpConfigIntf {ipaddr} {

    set intf ""
    set ip_route_output [exec_show_commands "show ip route $ipaddr"]
    if {([regexp {% Subnet not in table} $ip_route_output] == "0") || \
        ([regexp {% Network not in table} $ip_route_output] == "0")} {
        set a [split $ip_route_output "\n"]
        for {set i 0} {$i < [llength $a]} {incr i} {
            if {[regexp {Routing Descriptor Blocks:} [lindex $a $i]] == 1} {
                set b [lindex $a [expr $i+1]]
                set c [split $b " "]
                for {set j 0} {$j < [llength $c]} {incr j} {
                    if {[regexp {via} [lindex $c $j]] == 1} {
                        set intf [lindex $c [expr $j+1]]
                    }
                }                                        
            }
        }
    }
    return $intf

}    

###############################################################################
#
#    Description: Determines if access list is configured on the interface
#                 that is used for BGP Connection
#
#    IN: IP address of the neighbor
#    OUT: Returns message string "configured" if access list is configured and
#         "not configured" if the access list is not configured on the
#         interface that is used for BGP connection
#
################################################################################

proc checkAccessListConfig {ipaddr} {

    set msg "not configured"
    set bgp_intf ""
    set intf [findBgpConfigIntf $ipaddr]
    if {[regexp { [0-9]+\.[0-9]+\.[0-9]+\.[0-9]+ } $intf] == 1} {
        set bgp_intf [findBgpConfigIntf $intf]
    }

    if {$bgp_intf != ""} {
        set intf $bgp_intf
    }

    set output [exec_show_commands "show run interface $intf"]
    if {$output != ""} {
        foreach line [split $output "\n"] {
            if {[regexp { ip access-group [0-9]*} $line a] == 1} {
                set msg "configured"
            }
        }
    }
    return $msg    
}

################################################################################
#
#    Description: Determines if the neighbor is reachable through ping
#
#    IN: IP address of the neighbor
#    OUT: Returns "1" if ping is successful or "0" for unsuccessful ping
#    
################################################################################

proc exec_ping {ipaddr} {

    set ping_output [exec_show_commands "ping $ipaddr"]
    regexp {[0-9]+ percent} $ping_output a
    set percent [split $a " "]

    if {[lindex $percent 0] > 30 } {
         return 1
    } else {     
        return 0
    }
}

# Execute "show ip bgp neighbor" and check the output. Parse the output of the  
# show command and grep for the string "Last reset due to BGP Notification sent,
# hold time expired" and ping the neighbor 

    set parsed_output [exec_show_commands "show ip bgp neighbor $ip_address | i Last reset"]
    if {[regexp {Last reset ([0-9][0-9]):([0-9][0-9]):([0-9][0-9]), due to BGP Notification sent, hold time expired} $parsed_output] == 1} {
        if {[exec_ping $ip_address] == "1"} {
            append messageString "Neighbor $ip_address has gone down.\
            However, configured neighbor is reachable through ping. Check BGP\
            configuration changes at the peer and any access-list\
            restrictions between the peers.\n"
        } else {
            if {[checkAccessListConfig $ip_address] == "configured"} {
                append messageString "Neighbor $ip_address has gone down\
                and does not seem to be reachable through ping. Check BGP\
                configuration at the peer and any access-list restrictions\
                between the peers.\n"
            } else {
                append messageString "Neighbor $ip_address has gone down\
                and does not seem to be reachable through ping. Check network\
                connectivity.\n"
            }
        }
    } elseif {[regexp {Last reset ([0-9][0-9]):([0-9][0-9]):([0-9][0-9]), due to BGP Notification received, hold time expired} $parsed_output] == 1} {
        if {[exec_ping $ip_address] == "1"} {     
            append messageString "Neighbor $ip_address is reachable through\
            ping. Check BGP configuration at the peer and any access-list\
            restrictions between the peers.\n"
        } else {
            append messageString "Neighbor $ip_address has gone down and does\
            not seem to be reachable through ping. Check network connectivity,\
            BGP configuration at peer and any access-list restrictions between\
            the peers.\n" 
        }
    } else {
        if {[regexp {Last reset ([0-9][0-9]):([0-9][0-9]):([0-9][0-9]), due to Interface flap} $parsed_output a] == 1} {
            append messageString "Neighbor $ip_address has gone down and does\
            not seem to be reachable through ping. Check network\
            connectivity.\n"
        } else {
            append messageString "Neighbor $ip_address has gone down. But the\
            reason is not known at this time.\n"
        }
    }

set configHistory ""
set eventHistory ""

### TO LOG CONFIGURATION HISTORY ###

if {$_svcdiag_bgpLossConfigHistory == "TRUE"} {
    append configHistory \n\n " THE CONFIGURATION HISTORY is:\n "
    append configHistory [exec_show_commands "sh archive config diff"]
    append configHistory "-------------------------------------------------------------------------------"
}

### TO LOG EVENT HISTORY ###

if {$_svcdiag_bgpLossEventHistory == "TRUE"} {
    append eventHistory \n\n " THE EVENT HISTORY is :\n "
    append eventHistory [exec_show_commands "sh logging"]
    append eventHistory "-------------------------------------------------------------------------------"
}

### If the notification is not email send syslog message ###

if {$_svcdiag_bgpLossNotif == "syslog"} {
    action_syslog msg $messageString 
} elseif {$_svcdiag_bgpLossNotif == "email"} {
    set send_email 1
    append messageString $configHistory
    append messageString $eventHistory
} else {    
    if {$_svcdiag_bgpLossNotif == "all"} {
        action_syslog msg $messageString
        set send_email 1
        append messageString $configHistory
        append messageString $eventHistory
    }
}

###  send the email out ###

if {$send_email == "1"} {
    set routername [info hostname]
    if {[string match "" $routername]} {
        error "Host name is not configured"
    }

    set output [exec_show_commands "show event man dir user lib"]
    if {[regexp "not configured" $output match]} {
        error "User library directory is not configured"
    }
    set user_lib_dir [lindex $output 0]

    if [catch {smtp_subst [file join $user_lib_dir email_template]} result] {
        error $result $errorInfo
    }
    
    if [catch {smtp_send_email $result} result] {
        error $result $errorInfo
    } else {
        if {$_automation == "TRUE"} {
            puts $messageString
        }
    }
}
