::cisco::eem::event_register_timer watchdog name bgp_neighbor_route sec $_svcdiag_bgpRouteTimer maxrun 120 

#------------------------------------------------------------------------------
# EEM policy to check "BGP Route problems" which includes "route loss" "route 
#                      origination" and "route advertisement"
#
# June 2007 - Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007-2009 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-06/08/2007 
# Title: Report all the problems related to BGP route loss, origination and
#        advertisement to the customer
# Name: bgp_neighbor_route.tcl
# Desc: This script troubleshoots the causes for route problems due to missing
#       next-hop-self command, synchronization turned on and mis-match in
#       prefixes between routing table and bgp table  
# 
#       The structure and requirment of the file is defined below.
#
#       For the purpose of test automation an environment variable
#       _automation is required. This environment variable is checked if
#       defined previously and if not is defined.
# 
# Input-file:
#
#
# Environment Variables for this script are:
# _svcdiag_bgpRouteTimer
# _svcdiag_bgpRouteConfigHist
# _svcdiag_bgpRouteEventHistory
# _svcdiag_bgpRouteNotif
# _automation
# 
#------------------------------------------------------------------------------

# check if all the environment variables we need exist
# If any of them doesn't exist, print out an error message and quit

if {($_svcdiag_bgpRouteNotif == "email") || ($_svcdiag_bgpRouteNotif == "all")} {
    if {![info exists _email_server]} {
        set result \
            "Policy cannot be run: variable _email_server has not been set"
        error $result "_email_server is required for sending email"
    }

    if {![info exists _email_from]} {
        set result \
            "Policy cannot be run: variable _email_from has not been set"
        error $result "_email_from is required for sending email"
    }

    if {![info exists _email_to]} {
        set result \
            "Policy cannot be run: variable _email_to has not been set"
        error $result "_email_to is required for sending email"
    }

    if {![info exists _email_cc]} {
        #_email_cc is an option, must set to empty string if not set.
        set _email_cc ""
    }
}

if {![info exists _automation]} {
    set _automation FALSE
}
    
namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

global email bgpPrefixes validRoutes
set email 1
set send_email 0

#############################################################################
#
#    Description: Executes "show ip bgp", parses the output for the different
#                 networks and returns it 
#
#    IN: NONE 
#    OUT: Returns the different networks from the "show ip bgp" output
#  
#############################################################################

proc parseShowIpBgp {} {

    set network {}
    set output [exec_show_commands "sh ip bgp"]

    if {$output != ""} {
        foreach line [split $output "\n"] {
            if {[regexp {[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/[0-9]+} $line a] == 1} {
                if {[regexp { [0-9]+\.[0-9]+\.[0-9]+\.[0-9]+ } $line b] == 1} {
                    set word [split $line " "]
                    lappend network [lindex $word 0],[lindex $word 1],$b 
                }
            }
        }
    }

    return $network 
}

#############################################################################
#
#    Description: Executes "show ip route", parses the output for the BGP
#                 routes in the routing table and returns it 
#
#    IN: NONE 
#    OUT: Returns the BGP routes from the "show ip route" output
#  
#############################################################################

proc parseShowIpRoute {} {
   
    set output [exec_show_commands "sh ip route"]
    set bgproute {}

    foreach line [split $output "\n"] {
        if {[regexp {[A-Z]       [0-9]+\.[0-9]+\.[0-9]+\.[0-9]+} $line] == 1} {
            if {[regexp {B       [0-9]+\.[0-9]+\.[0-9]+\.[0-9]+} $line] == 1} {
                set network [split $line " "]
                lappend bgproute [lindex $network 7] 
            }
        }       
    }
    
    return $bgproute
}

#############################################################################
#
#    Description: From a list of bgp prefixes finds out just the valid routes  
#                 and returns it 
#
#    IN: A list of bgp prefixes 
#    OUT: Returns a list of valid routes 
#  
#############################################################################

proc findIfNotBestPathRoutes {nlist} {

    set valid {}

    for {set i 0} {$i < [llength $nlist]} {incr i} {
        set route [split [lindex $nlist $i] ,]
        set temp [lindex $route 0]
        if {$temp != "*>"} {
            if {($temp == "*") || ($temp == "r>") } {
                lappend valid [lindex $route 1],[lindex $route 2]
            }
        }
    }
    
    return $valid
} 

#############################################################################
#
#    Description: Executes show run and parses the output to look for      
#                 synchronization configuration if turned on or not and returns
#                 appropriate error message. 
#
#    IN: None 
#    OUT: Returns error message if synchronization is turned on else returns 
#         the error message that synchronization is turned off. 
#  
#############################################################################

proc checkSynchronization {} {

    set msgstr ""
    set show_run [show_run_section "router bgp"]

    if {$show_run != ""} {
        set bgp_config [split $show_run "\n"] 
        set config [lindex $bgp_config 1]
        if {[regexp {no synchronization} $config] == 1} {
            append msgstr ""
        } elseif {[regexp {synchronization} $config] == 1} {
            append msgstr "Synchronization is turned on in the BGP \
            configuration. Prefixes might not be installed in the Routing \
            table if they are not known through IGP\n\n"
        } 
    }

    return $msgstr
}

#############################################################################
#
#    Description: Executes ping of the ipaddress that is given as input and   
#                 returns 0 or 1 based on the success of the ping 
#
#    IN: ip address on which ping has to be executed 
#    OUT: Returns 0 if the ping fails otherwise returns 1 
#  
#############################################################################

proc ping {ipaddr} {

    set output [exec_show_commands "ping $ipaddr"]
    regexp {[0-9]+ percent} $output a
    set percent [split $a " "]

    if {[lindex $percent 0] > 30 } {
        return 1
    } else {
        return 0
    }
}

#############################################################################
#
#    Description: Executes ping of the next hop ip address that is passed as
#                 input and returns error message if next hop is not reachable 
#
#    IN: list of valid routes and their next hop ip address 
#    OUT: Returns error message if the next hop is not reachable otherwise 
#         returns null string. 
#  
#############################################################################

proc checkNextHopReachability {vlist} {

    set msgstr ""
    
    for {set i 0} {$i < [llength $vlist]} {incr i} {
        set nextHop [split [lindex $vlist $i] ,]
        if {[ping [lindex $nextHop 1]] == 0} {
            append msgstr "BGP Next-hop [lindex $nextHop 1] of bgp prefix\
                           [lindex $nextHop 0] is not reachable as ping fails;\
                           next-hop-self need to be configured in the neighbor\
                           router\n"
        }
    }

    return $msgstr
}

#############################################################################
#
#    Description: Given a number, computes and returns the binary equivalent
#                 of the input 
#
#    IN: Number corresponding to one octet of the subnet mask   
#    OUT: Returns binary equivalent of the octet 
#  
#############################################################################

proc computeSubnet {num} {

    set binary ""

    while {$num != 0} {
        append binary [expr $num % 2]
        set num [expr $num / 2] 
    }

    return $binary
}

#############################################################################
#
#    Description: Given the subnet mask returns the binary equivalent of it
#                 
#    IN: Number corresponding to the subnet mask   
#    OUT: Returns binary equivalent of the subnet mask 
#  
#############################################################################

proc findSubnetValue {mask} {
    
    set octet [split $mask "."]

    append value [computeSubnet [lindex $octet 0]]
    append value [computeSubnet [lindex $octet 1]]
    append value [computeSubnet [lindex $octet 2]]
    append value [computeSubnet [lindex $octet 3]]

    return $value
}

#############################################################################
#
#    Description: Executes "show ip route" and parses the output to get the 
#                 subnet mask. The mask is compared with the mask in the BGP 
#                 table 
#                 
#    IN: List of networks from the BGP table   
#    OUT: Returns error message if the subnet masks do not match else returns
#         null string 
#  
#############################################################################

proc checkRoutingTable {rlist} {

    set msgstr ""
    set network {}
    
    for {set i 0} {$i < [llength $rlist]} {incr i} {
        set netmask [split [lindex $rlist $i] ","]
        set output [exec_show_commands "sh ip route [lindex $netmask 0] "]
        if {[regexp {[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/[0-9]+} $output a] == 1} {
            set network [split $a "/"]
            if {[lindex $network 1] != [string length [findSubnetValue \
                [lindex $netmask 1]]]} {
                append msgstr "There is a mismatch between prefix mask in \
                    the IP routing table and BGP configuration "
                append msgstr "network [lindex $netmask 0] mask \
                    [lindex $netmask 1]\n"
            }
         }
    }

    return $msgstr
}

#############################################################################
#
#    Description: Executes "show ip bgp" of all neighbors in the bgp prefix 
#                 list. Appends all the prefixes that do not give any output,
#                 to a list and invokes the procedure "checkRoutingTable" if
#                 the list is not empty 
#                 
#    IN: List of BGP prefixes  
#    OUT: Calls the procedure "checkRoutingTable"  
#  
#############################################################################

proc checkBgpTable {blist} {
    
    global email 
    set network {}

    for {set i 0} {$i < [llength $blist]} {incr i} {
        set netmask [split [lindex $blist $i] ","]
        set output [exec_show_commands "sh ip bgp [lindex $netmask 0] \
                                                  [lindex $netmask 1]"]
        if {[regexp {% Network not in table} $output] == 1} {
            lappend network [lindex $blist $i]
        }
    }
    
    if {[llength $network] == 0} {
        set email 0
        return ""
    } else {
        set email 1
        return [checkRoutingTable $network]
    }
}

#############################################################################
#
#    Description: Looks for symptoms related to route loss. If the bgp routes
#                 are just valid routes then checks for synchronization 
#                 configuration and next hop reachability and returns
#                 appropriate error message 
#                 
#    IN: None  
#    OUT: Returns error message if synchronization is turned on or if next hop
#         is reachable.   
#  
#############################################################################

proc routeLoss {} {

    global email bgpPrefixes validRoutes
    set msgstr ""
    set bgpPrefixes {}
    set validRoutes {}

    set bgpPrefixes [parseShowIpBgp]
    set ipRoutes [parseShowIpRoute]
    if {[llength $bgpPrefixes] != 0} {
        set validRoutes [findIfNotBestPathRoutes $bgpPrefixes]
    }

    if {[llength $validRoutes] != 0} { 
        append msgstr [checkSynchronization]
        append msgstr [checkNextHopReachability $validRoutes] 
    } else {
        set email 0
    }

    return $msgstr
}

#############################################################################
#
#    Description: Procedure to massage the configuration "network *.*.*.* 
#                 mask *.*.*.*" and return network-mask pair 
#                 
#    IN: The network configuration  like "network *.*.*.* mask *.*.*.*" 
#    OUT: Returns the network-mask pair 
#  
#############################################################################

proc networkMaskPairForRouteOrigination {network_config} {

    set net $network_config
    for {set i 0} {$i < [llength $net]} {incr i} {
        set litem [lindex $net $i] 
        if {[regexp {[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+} $litem a] == 1} { 
            if {([lindex $net $i] > 0) && \
                ([lindex $net [expr $i-1]] == "network")} {
                set network [lindex $net $i]  
            } else {
                if {([lindex $net $i] > 0) && \
                    ([lindex $net [expr $i-1]] == "mask")} {
                    set mask [lindex $net $i]
                }
            } 
        }
    }
    return $network,$mask
}

#############################################################################
#
#    Description: Procedure to look for a list of available networks. Executes
#                 "show run" and parses the output for network configurations.
#                 
#    IN: None  
#    OUT: Calls the procedure "checkBgpTable" with the network parsed from the 
#         show run output.   
#  
#############################################################################

proc routeOrigination {} {

    set network {}
    set output [show_run_section "router bgp"]

    foreach line [split $output "\n"] {
        if {[regexp {network *.*.*.* mask *.*.*.*} $line a] == 1} {
            set net [split $line " "]
            lappend network [networkMaskPairForRouteOrigination $net]
        } else {
            if { [regexp {network *.*.*.*} $line b] == 1} {
                string trimright $b
                regsub -all {[ \t\r\n]+} $b " " out
                set ip [split $out " "]
                set network_ip [lindex $ip 1]
                set mask [find_class_subnet $network_ip] 
                lappend network $network_ip,$mask 
            }
        }
    }

    if {[llength $network] != 0} {
        return [checkBgpTable $network]
    }
}

#############################################################################
#
#    Description: Procedure to parse the output of "show ip bgp neighbor
#                 x.x.x.x advertised-routes" and look for the number of
#                 prefixes. 
#                
#    IN: Output of "show ip bgp neighbor x.x.x.x advertised-routes" 
#    OUT: Returns the number of prefixes 
# 
#############################################################################

proc find_if_prefixes_is_zero {output} {

    set prefixes ""
    if {$output != ""} {
        if {[regexp {Total number of prefixes [0-9]+} $output a] == 1} {
            regsub -all {[ \t]+} $a " " out
            string trim $out
            set temp [split $out " "]
            set prefixes [lindex $temp 4]
        }
    }
    return $prefixes
}

#############################################################################
#
#    Description: Procedure to check for route advertisement problems. Executes 
#                 "show ip bgp neighbor advertised-routes" and based on the 
#                 output returns error message if the routes are advertised
#                 or not.  
#                 
#    IN: None  
#    OUT: Returns error message if routes had been advertised or not with the 
#         reason for not advertising   
#  
#############################################################################

proc routeAdvertisement {} {

    global bgpPrefixes validRoutes email 
    set msgstr ""
    set output [exec_show_commands "show ip bgp summary"]
    set keywords "Neighbor MsgSent State/PfxRcd"
    set output [parse_table $output $keywords]
    set output [parse_header_and_values $output]
    set neighbor {}
    set bestpath "false"

    set table_value_len [llength $output]
    if {$table_value_len != 0} {
        for {set i 0} {$i < $table_value_len} {incr i} {
            set row [split [lindex $output $i] " "]
            lappend neighbor [lindex $row 0]
        }
    }

    if {[llength $neighbor] != 0} {
        for {set i 0} {$i < [llength $neighbor]} {incr i} {
            set neigh [lindex $neighbor $i]
            set output [exec_show_commands "sh ip bgp neighbor $neigh \
                                           advertised-routes "] 
            if {[find_if_prefixes_is_zero $output] != "0"} {
                    set bestpath "true" 
            } else {
                set output [show_run_section "router bgp"]
                if {[regexp "network *.*.*.*" $output a] != 1} {
                    # do nothing
                } else {
                    append msgstr "Neighbor $neigh is not being advertised \
                                   any routes\n"
                }
            }
        }
    }
 
    set len [llength $validRoutes]
    if {$len != 0} {
        if {$bestpath != "true"} {
            append msgstr "THE ROUTES:\t"
            for {set i 0} {$i < [llength $validRoutes]} {incr i} {
                 set temp [split [lindex $validRoutes $i] , ]
                 append msgstr "[lindex $temp 0] \t"         
            }
            append msgstr "which are not marked as best path prefixes in the BGP\
                           table are not installed in the routing table and so\
                           will not be advertised to peer"
        } else {
            append msgstr "Best path route exists, Check for other reasons why\
                           Routes are not advertised.\n"   
        }
    } else {
        set email 0
    }
    return $msgstr
}

### Execution of the scripts start from here ###

set messageString "\nTHE SERVICE DIAGNOSTIC MESSAGE FOR BGP NEIGHBOR ROUTE PROBLEM IS :\n"
set configHistory ""
set eventHistory ""

### Call for procedures which troubleshoots the different route problems ###

set route_loss_msg [routeLoss]
set route_orig_msg [routeOrigination]
set route_adv_msg [routeAdvertisement]

if {($route_loss_msg != "") || ($route_orig_msg != "") || ($route_adv_msg != "")} {
    set email 1
}

append messageString $route_loss_msg
append messageString $route_orig_msg
append messageString $route_adv_msg

### TO LOG CONFIGURATION HISTORY ###

if {$_svcdiag_bgpRouteConfigHist == "TRUE"} {
    set configHistory "\n\n  THE CONFIGURATION HISTORY is:\n " 
    append configHistory [exec_show_commands "sh archive config diff"]
    append configHistory "-------------------------------------------------------------------------------"
}

### TO LOG EVENT HISTORY ###

if {$_svcdiag_bgpRouteEventHistory == "TRUE"} {
    set eventHistory "\n\n  THE EVENT HISTORY is :\n "
    append eventHistory [exec_show_commands "sh logging"]
    append eventHistory "-------------------------------------------------------------------------------"
}

set output [exec_show_commands "show event man dir user lib"]
if {[regexp "not configured" $output match]} {
    error "User library directory is not configured"
}
set user_lib_dir [lindex $output 0]

if {$email == "1"} {
    if {$_svcdiag_bgpRouteNotif == "syslog"} {
        action_syslog msg $messageString
    } elseif {$_svcdiag_bgpRouteNotif == "all"} {
        set send_email 1
        action_syslog msg $messageString
        append messageString $eventHistory
        append messageString $configHistory
    } else {
        if {$_svcdiag_bgpRouteNotif == "email"} {     
            set send_email 1
            append messageString $eventHistory
            append messageString $configHistory
        }
    }
} 

### send the email out ###

if {$email == "1"} {
    if {$send_email == "1"} {
        set routername [info hostname]
        if {[string match "" $routername]} {
            error "Host name is not configured"
        }

        if [catch {smtp_subst [file join $user_lib_dir email_template]} result] {
            error $result $errorInfo
        }

        if [catch {smtp_send_email $result} result] {
            error $result $errorInfo
        } else {
            if {$_automation == "TRUE"} {
                puts $messageString
            }
        }
    }
}
