
#------------------------------------------------------------------------------
# Diagnostic script to get the inputs related to "BGP Neighbor Loss Problem",
#     "BGP Neighbor Formation Problem" and  "BGP Route Problem" from the user
#
# October 2007 - Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007, 2009 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-10/17/2007
# Title: Store(configure) inputs from user and register with EEM.
# 
# Name: sdiag_bgp.tcl
#
# Desc: Gets input from customer, validates them and and stores(configures on
#       the router) the inputs. Registers the  corresponding BGP policies
#       "bgp_neighbor_loss.tcl", "bgp_neighbor_formation.tcl" and "bgp_
#       neighbor_route.tcl" with EEM
#
#       The inputs will be collected from user non-interactively
#
# Usage: tclsh disk#:/dir_name/sdiag_bgp.tcl <bgp policy name> <notification>
#        <configurationHistory> <EventHistory> <userPolicyDirectory>
#        <userLibraryDirectory> <additionalInputs> where
#
#        The <bgp policy name> -- ["bgp_neighbor_loss.tcl" or
#            "bgp_neighbor_formation.tcl" or "bgp_neighbor_route.tcl"]
#            <notification> -- ["email" or "syslog"] 
#            <configurationHistory> -- ["TRUE" or "FALSE"]
#            <EventHistory>  -- ["TRUE" or "FALSE"]
#            <userPolicyDirectory> -- [absolute path of policy directory]
#            <userLibraryDirectory> -- [absolute path of library directory]
#            <additionalInputs> -- [timer value in seconds between the range
#                                   1 and 4294967295]   
#        
#-----------------------------------------------------------------------------        
#############################################################################
#
#    Description: Checks if the user library directory is configured and 
#                 registered with EEM
#
#    IN :None
#    OUT:sources the files userlib.tcl and diag_lib.tcl 
#
#############################################################################    
proc sourceLibFile {user_lib} {
   
    if {[file exists [file join $user_lib diag_lib.tcl]]} {
        set user_lib_dir $user_lib
    } else {
        set user_lib_dir [exec show event man dir user lib]
        if {[regexp "not configured" $user_lib_dir match]} {
            puts "SERVICE DIAGNOSTICS ERROR: User library directory is \
            not configured nor does specified path exist."
            return 0
        }
    }

    source [file join $user_lib_dir diag_lib.tcl]
    source [file join $user_lib_dir userlib.tcl]
    return 1

}

#############################################################################
#
#    Description: Registers the user policy directory and user library
#                 directory with EEM
#
#    IN: "type" which is either "policy" or "library" and "dir_name" is the
#        name of the policy directory or name of the library directory 
#    OUT:Gives error message whether the registration of user policy directory
#        or user library directory was successful or not.   
#
##############################################################################                             
proc set_eem_dir { type dir_name } {
    
    if {[catch {ios_config "event manager directory user $type $dir_name"} err]} {
        set err [string trim $err]
        puts "err is \"$err\" --- Failed to config EEM user $type directory"
    }
}
                                            
#############################################################################
#
#    Description: If the notification is email then we check if the email
#                 related inputs have been given by the user previously,
#                 otherwise an error message is sent to the user to give the
#                 inputs related to email
#
#    IN: None    
#    OUT:  Invokes the file collectEmailParameters.tcl if the email inputs
#          were not given initially
#
##############################################################################

proc configure_email {notif} {

    set err 0
    
    if {($notif == "email") || ($notif == "all")} {    
        if {[emailInputAlreadyGiven  $notif] != "configured"} {
            puts "\nNo inputs corresponding to email was collected"
            puts "Run the file 'collectEmailParameters.tcl'"
            puts "Usage: tclsh disk#:/dir_name/collectEmailParameters.tcl email_fromID email_toID email_ccID email_server name "        
            set err 1
        }   
    }    
    
    return $err
    
}

#############################################################################
#
#    Description: Configure EEM environment variables for BGP neighbor loss
#                 scenario 
#
#    IN: The inputs given by the user  
#    OUT:  Configures the inputs as  environment variables on the router 
#      
###############################################################################

proc loss_config {notif config event} {
    
    set_eem_env _svcdiag_bgpLossNotif $notif
    set_eem_env _svcdiag_bgpLossConfigHistory $config
    set_eem_env _svcdiag_bgpLossEventHistory $event
    unreg_eem_policy bgp_neighbor_loss.tcl
    reg_eem_policy bgp_neighbor_loss.tcl
    
}

#############################################################################
#
#    Description: Configure EEM environment variables for BGP neighbor 
#                 formation scenario 
#
#    IN: The inputs given by the user  
#    OUT:  Configures the inputs as  environment variables on the router 
#      
################################################################################

proc formation_config {notif config event time} {
    
    set_eem_env _svcdiag_bgpFormTimer $time
    set_eem_env _svcdiag_bgpFormNotif $notif
    set_eem_env _svcdiag_bgpFormConfigHistory $config
    set_eem_env _svcdiag_bgpFormEventHistory $event
    unreg_eem_policy bgp_neighbor_formation.tcl
    reg_eem_policy bgp_neighbor_formation.tcl
    
}

#############################################################################
#
#    Description: Configure EEM environment variables for BGP neighbor
#                 route problem scenario
#
#    IN: The inputs given by the user
#    OUT:  Configures the inputs as  environment variables on the router
#
################################################################################                 

proc route_config {notif config event time} {

    set_eem_env _svcdiag_bgpRouteTimer $time
    set_eem_env _svcdiag_bgpRouteNotif $notif
    set_eem_env _svcdiag_bgpRouteConfigHist $config
    set_eem_env _svcdiag_bgpRouteEventHistory $event
    unreg_eem_policy bgp_neighbor_route.tcl
    reg_eem_policy bgp_neighbor_route.tcl
    
}

###############################################################################
#
#    Description: This procedure sends a warning message to user if the user
#                 wants the output of configuration history and event history
#                 to be sent along with the syslog message
#
#    IN : Notification, configuration history option and event history option
#    OUT: Sends a warning message that the configuration history or event
#         history will not be sent along with syslog message
#         
###############################################################################         
proc send_sdiag_warning {notif config event} {

    if {($notif == "syslog") || ($notif == "all")} {
        if {($config == "TRUE") || ($event == "TRUE")} {
            puts "\n\nSERVICE-DIAGNOSTICS MESSAGE:"
            puts "\nConfiguration history or Event history cannot be sent
            along with syslog notification\n"
        }
    }
}

#    Execution of the script starts from here
#    Check if the number of arguments entered by user is correct otherwise send
#    error message. 
              
set error "\nFOR BGP NEIGHBOR LOSS PROBLEM"
append error "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_bgp.tcl <bgp_neighbor_loss.tcl> <notification> <configurationHistory> <EventHistory> <user_pol_dir> <user_lib_dir>"
append error "\n"
append error "\nFOR BGP NEIGHBOR FORMATION PROBLEM"
append error "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_bgp.tcl <bgp_neighbor_formation.tcl> <notification> <configurationHistory> <EventHistory> <user_pol_dir> <user_lib_dir> <timer>"
append error "\n"
append error "\nFOR BGP NEIGHBOR ROUTE PROBLEM"
append error "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_bgp.tcl <bgp_neighbor_route.tcl> <notification> <configurationHistory> <EventHistory> <user_pol_dir> <user_lib_dir> <timer>"

if {($argc == 6) || ($argc == 7)} {
    set notif [lindex $argv 1]
    set config [lindex $argv 2]
    set event [lindex $argv 3]
    set user_pol_dir [lindex $argv 4]
    set user_lib_dir [lindex $argv 5]

    if {[sourceLibFile $user_lib_dir] == 0} {
        return
    }
    unset_eem_dir "policy"
    unset_eem_dir "library"
    set_eem_dir policy $user_pol_dir
    set_eem_dir library $user_lib_dir

} else {
    puts "\n\nSERVICE DIAGNOSTICS ERROR: No correct number of arguments"
    puts $error
    return
}

switch [lindex $argv 0] {

    bgp_neighbor_loss.tcl {
        
        if {$argc == 6} {
            
            if {[validateOtherInputs $notif $config $event] == "1"} {
                return
            }
            
            send_sdiag_warning $notif $config $event

        } elseif {$argc == 0} {
            if {$tcl_interactive == "1"} {
            } else {
                 puts "\nSERVICE-DIAGNOSTICS ERROR: No input was given"
                 puts $error
                 return
            }
        } else {
            puts "\n\nSERVICE DIAGNOSTICS ERROR: Check Usage"
            puts $error
            return
        }

    #   Get email inputs and configure if they don't exist  
        
        if {[configure_email $notif] == 1} {
            return 
        }

    # Call the procedure loss_config to do the eem configuration    
          
        loss_config $notif $config $event
    }
    
    bgp_neighbor_formation.tcl {
        
        if {$argc == 7} {
            set timer [lindex $argv 6]
            set validation1 [validateOtherInputs $notif $config $event]
            set validation2 [validateEEMPolicyTimer $timer]
        
            if {($validation1 == "1") || ($validation2 == "1")} {
                return
            }
            set timer $validation2            
            send_sdiag_warning $notif $config $event
            
        } elseif {$argc == 0} {
            if {$tcl_interactive == "1"} {
            } else {
                puts "\n\nSERVICE-DIAGNOSTICS ERROR: No input was given"
                puts $error
                return
            }
        } else {
            puts "\n\nSERVICE DIAGNOSTICS ERROR: Check Usage"
            puts $error
            return
        }
    #   Get email inputs and configure if they don't exist 
        
        if {[configure_email $notif] == 1} {
            return
        }

    # Call the procedure formation_config to do the eem configuration
        
        formation_config $notif $config $event $timer 
    }
    
    bgp_neighbor_route.tcl {
        
        if {$argc == 7} {
            set timer [lindex $argv 6]
            set validation1 [validateOtherInputs $notif $config $event]
            set validation2 [validateEEMPolicyTimer $timer]
        
            if {($validation1 == "1") || ($validation2 == "1")} {
                return
            }
            set timer $validation2

            send_sdiag_warning $notif $config $event
            
        } elseif {$argc == 0} {
            if {$tcl_interactive == "1"} {
            } else {
                puts "\n\nSERVICE-DIAGNOSTICS ERROR: No input was given"
                puts $error
                return
            }
        } else {
            puts "\n\nSERVICE DIAGNOSTICS ERROR: Check Usage"
            puts $error
            return
        }    

        #   Get email inputs and configure if they don't exist
           
            if {[configure_email $notif] == 1} {
                return
            }

        # Call the procedure route_config to do the eem configuration    
            
            route_config $notif $config $event $timer 
        }   

    default {
        puts "\n\nSERVICE DIAGNOSTICS ERROR : $error"
    }     
}


#Cisco Tcl Signature V1.0
#3082077506092a864886f70d010702a082076630820762020101310b3009
#06052b0e03021a0500300b06092a864886f70d010701a08204bb308204b7
#3082039fa003020102020900d48b1bbfb069e9b9300d06092a864886f70d
#0101050500308198310b3009060355040613024341311230100603550408
#130963616c69666f6e69613111300f060355040713086d69706c69746173
#311b3019060355040a1312636973636f2073797374656d7320696e632e31
#0e300c060355040b13054e53535447311630140603550403130d43687269
#7374696e652059616e311d301b06092a864886f70d010901160e6379616e
#40636973636f2e636f6d301e170d3037313231383138333232375a170d31
#30313231373138333232375a308198310b30090603550406130243413112
#30100603550408130963616c69666f6e69613111300f060355040713086d
#69706c69746173311b3019060355040a1312636973636f2073797374656d
#7320696e632e310e300c060355040b13054e535354473116301406035504
#03130d436872697374696e652059616e311d301b06092a864886f70d0109
#01160e6379616e40636973636f2e636f6d30820122300d06092a864886f7
#0d01010105000382010f003082010a02820101009c22f1d01af397756ab6
#cfce92a9e761b73fbf1efd070aa4e90448e6077e2c127ff47a5951de7790
#55da014ff152576a08d7199b8ad5757eb8c9a7c1a493715df2176e9f56f0
#74735d43420cf1ed17fe0aa3494724808aefcf274572c7750be071a94560
#9c045c48a5454f9e7933bf93d967bcf582918b1bbd48f47dd978c7ea9c88
#23eb01482a40ef770f823ea2e63eaf7cab5367bd51b54979925464e27b22
#82fc5bfe94f6f41f2f91e56444ae371a050107fe79fd5db3f228bbbd5fcb
#653a87a3c1268d74500ba1489382290804d9d7937d3269b1c55eae51f9f7
#62eadbd4226ba6d4008414b353181ee6cfbe24204e4a82436b117c255014
#77ec74c3ac4d0203010001a38201003081fd301d0603551d0e041604149c
#0e12d3a222891fc1300b780f38b54e07e228073081cd0603551d230481c5
#3081c280149c0e12d3a222891fc1300b780f38b54e07e22807a1819ea481
#9b308198310b300906035504061302434131123010060355040813096361
#6c69666f6e69613111300f060355040713086d69706c69746173311b3019
#060355040a1312636973636f2073797374656d7320696e632e310e300c06
#0355040b13054e53535447311630140603550403130d436872697374696e
#652059616e311d301b06092a864886f70d010901160e6379616e40636973
#636f2e636f6d820900d48b1bbfb069e9b9300c0603551d13040530030101
#ff300d06092a864886f70d010105050003820101003e6d657e30c21acb30
#d7e91aef99cd6f83a8644d7ae437fb4a0cd1a8c6a0f1cb2194be80868b75
#1d20ee8dc72a1f80de697928820ec3f806cec67df136d6f8b6ff0ba7ef6f
#e4c228b37e7407becff9903198ebe2258d3774f198de8d0366cb10496408
#57dff62f247dfcae9813fde4167c1f959b291490cfb8e0b8db3395ce019f
#07c94aa4a86d1a0dae2b6cc6d139db740841dc65add58512ea59336dbc8c
#9abc14d500e42c8ea5031c201eec027d14e9878974629d2fdbe0d0cfffab
#8ab73f148bdec788de59550443ccf12cb3e74518274c3d7d2892eeb90f23
#0d5e853b85542c05b12198c097c2fb4023b370f0629645e11552f5429fb8
#aacef60a9344b5318202823082027e0201013081a6308198310b30090603
#55040613024341311230100603550408130963616c69666f6e6961311130
#0f060355040713086d69706c69746173311b3019060355040a1312636973
#636f2073797374656d7320696e632e310e300c060355040b13054e535354
#47311630140603550403130d436872697374696e652059616e311d301b06
#092a864886f70d010901160e6379616e40636973636f2e636f6d020900d4
#8b1bbfb069e9b9300906052b0e03021a0500a081b1301806092a864886f7
#0d010903310b06092a864886f70d010701301c06092a864886f70d010905
#310f170d3039303330353233333932345a302306092a864886f70d010904
#31160414bdd8328ad55cb1ade3cbddcdc1119b0474241877305206092a86
#4886f70d01090f31453043300a06082a864886f70d0307300e06082a8648
#86f70d030202020080300d06082a864886f70d0302020140300706052b0e
#030207300d06082a864886f70d0302020128300d06092a864886f70d0101
#01050004820100591026074d3adea7c2492347744dc11ab5730085256eba
#76a66a25bd712d488790a9d726251c00ca628cc0f1ee11857c2a89d9a16d
#7aaef93c1602f416dafd6d93b899af0dd915cd2cbf8ab3e7a036f799518f
#c4c644a71e5b8ca4af05535f579493bbcf248c5d1cece056b6f06b543f88
#2016c863992cd3bbce5fa2ea3ef08746fa557559ea38334ab20c5ec867d0
#d5f938de1460258fc909b0a43e59e8f0d273a68baa61b978b7c866ae2c70
#7358fa448c4dd6bcb1e0ab7ee3b1f4f44c67e9d60a849b6d89f8734ad9b1
#8a9686408055b3ea45618a60a2250732a7feb5bbc48770c030306459c871
#6c0713617dc871f94f570faaa0a0e97f40a78a6f245056
