set user_lib_dir [exec show event man dir user lib]
if {[regexp "not configured" $user_lib_dir match]} {
    error "User library directory is not configured"
}

### Sourcing the library file ###
source [file join $user_lib_dir userlib.tcl]
source [file join $user_lib_dir diag_lib.tcl]

### gloabl variable declaration ###
global show_run_output


#------------------------------------------------------------------------------
# Diagnostic script to undeploy the existing inputs and policies 
#
# July 2007 - Vimaladevi Rajakanthan (vrajakan@cisco.com)
#
# Copyright (c) 2007 by cisco Systems, Inc.
# All rights reserved.
#
#
# Version-date: v1.0-07/20/2007 
# Title: Undeploy the inputs, related to BGP and ERM policies
# 
# Name:  sdiag_undeploy.tcl
#
# Desc: Undeploys i.e. removes all the inputs related to the bgp policy file
#       passed as parameter. Also removes all the email related inputs if
#       the string "email" is passed as the parmeter. The inputs can be
#       collected from user non-interactively
#
# Usage: tclsh disk#:/dir_name/sdiag_bgp_undeploy.tcl <bgp policy> 
#                            OR   
#        tclsh disk#:/dir_name/sdiag_bgp_undeploy.tcl <email>
#        
#       The <bgp policy> -- ["bgp_neighbor_loss.tcl" or
#           "bgp_neighbor_formation.tcl" or bgp_neighbor_route.tcl]
#           <email> -- [a string "email"] 
#
#------------------------------------------------------------------------------

global error type

###############################################################################
#
#    Description: Gets the environment variable name parses it and sets error
#                 message corresponding to the BGP problem type
#
#    IN : environment variable name corresponding to BGP problem type
#    OUT: returns the error message which was set based on the input BGP
#         problem type
#
###############################################################################

proc get_bgp_prototype {type} {

    set error "\n\nSERVICE DIAGNOSTICS ERROR:\n"
    
    switch -regexp -- $type {

        Loss {
            append error "\nThe policy \"bgp_neighbor_loss.tcl\" and its related
 inputs were not deployed"
        }
        Form {
            append error "\nThe policy \"bgp_neighbor_formation.tcl\" and its related inputs were not deployed"
        }
        Route {
            append error "\nThe policy \"bgp_neighbor_route.tcl\" and its related inputs were not deployed"
        }
        email {
            append error "\nNo email inputs were given"
        }
        default {
            puts "No match"
        }
    }

    return $error
}

#############################################################################
#
#    Description: Executes show run command and parses the output for
#                 configuration corresponding to the input entered by the user
#                 and returns it  
#
#    IN: The unique keyword to look for in the show run output   
#    OUT: Returns a list of configurations on the router corresponding to the
#         keyword entered 
#  
#############################################################################

proc exec_show_run {keyword} {
   
    global show_run_output
    set output ""
    
    if {[catch {exec "show event man env"} show_run_output] != 0} {
        puts stderr "'show run' did not execute"
    } else {
        foreach line [split $show_run_output "\n"] {
            switch -regexp -- $keyword {
                bgpLoss {
                    if {[regexp {_svcdiag_bgpLoss} [lindex $line 1] a] == 1} {
                        lappend output $line
                    }
                }
                bgpForm {
                    if {[regexp {_svcdiag_bgpForm} [lindex $line 1] a] == 1} {
                        lappend output $line
                    }
                }
                bgpRoute {
                    if {[regexp {_svcdiag_bgpRoute} [lindex $line 1] a] == 1} {
                        lappend output $line
                    }
                }
                email {
                    if {[regexp {_email} [lindex $line 1] a] == 1} {
                        lappend output $line
                    }
                }
                default {
                    puts "no input"
                }
            }    
            ### end of switch ###
        }
        ### end of foreach ### 
        
        if {[llength $output] != 0} { 
            return $output
        } else {
            puts [get_bgp_prototype $keyword]
            return 1
        }
    }
}

#############################################################################
#
#    Description: Unconfigures the list of configuration that is passed to it 
#                 as input
#
#    IN: The list of configuration commands to be unconfigured   
#    OUT: NONE 
#  
#############################################################################

proc unconfigure {output} {

    set i 0
    
    while {$i < [llength $output]} {
        set var [lindex $output $i]
        ios_config "no event manager environment [lindex $var 1] [lindex $var 2]"
        set i [expr $i + 1]
    }
}

### Script Execution starts here ###

set error "\nFOR BGP SCRIPTS"
append error "\nUsage:tclsh <disk#:>/<dir_name>/<sdiag_bgp_undeploy.tcl> <bgp_neighbor_loss.tcl> OR <bgp_neighbor_formation.tcl> OR <bgp_neighbor_route.tcl>"
append error "\nOR"
append error "\nFOR EMAIL"
append error "\nUsage:tclsh <disk#:>/<dir_name>/sdiag_bgp_undeploy.tcl <email>" 

if {$argc == 0} {
    if {$tcl_interactive == "1"} {
    } else {
        puts "\n\nSERVICE-DIAGNOSTICS ERROR: No inputs were given"
        puts $error
        return
    }
} else {
    if {$argc > 1} {
        puts $error
        return
    }
}

if {$argc == 1} {
    set type [lindex $argv 0]
}

switch $type {
    
    bgp_neighbor_loss.tcl {
        set output [exec_show_run _svcdiag_bgpLoss]
        if {$output != "1"} {
            unconfigure $output
            unreg_eem_policy "bgp_neighbor_loss.tcl"
        } else return
    }
    
    bgp_neighbor_formation.tcl {
        set output [exec_show_run _svcdiag_bgpForm]
        if {$output != "1"} {
            unconfigure $output
            unreg_eem_policy "bgp_neighbor_formation.tcl"
        } else return
    }
    
    bgp_neighbor_route.tcl {
        set output [exec_show_run _svcdiag_bgpRoute]
        if {$output != "1"} {
            unconfigure $output
            unreg_eem_policy "bgp_neighbor_route.tcl"
        } else return
    }
    
    email {
        set output [exec_show_run _email]
        unconfigure $output
    }
    
    default {
        global error
        puts "\nSERVICE DIAGNOSTICS ERROR: Correct input was not entered"
        puts $error
    }
} 


#Cisco Tcl Signature V1.0
#3082077506092a864886f70d010702a082076630820762020101310b3009
#06052b0e03021a0500300b06092a864886f70d010701a08204bb308204b7
#3082039fa003020102020900d48b1bbfb069e9b9300d06092a864886f70d
#0101050500308198310b3009060355040613024341311230100603550408
#130963616c69666f6e69613111300f060355040713086d69706c69746173
#311b3019060355040a1312636973636f2073797374656d7320696e632e31
#0e300c060355040b13054e53535447311630140603550403130d43687269
#7374696e652059616e311d301b06092a864886f70d010901160e6379616e
#40636973636f2e636f6d301e170d3037313231383138333232375a170d31
#30313231373138333232375a308198310b30090603550406130243413112
#30100603550408130963616c69666f6e69613111300f060355040713086d
#69706c69746173311b3019060355040a1312636973636f2073797374656d
#7320696e632e310e300c060355040b13054e535354473116301406035504
#03130d436872697374696e652059616e311d301b06092a864886f70d0109
#01160e6379616e40636973636f2e636f6d30820122300d06092a864886f7
#0d01010105000382010f003082010a02820101009c22f1d01af397756ab6
#cfce92a9e761b73fbf1efd070aa4e90448e6077e2c127ff47a5951de7790
#55da014ff152576a08d7199b8ad5757eb8c9a7c1a493715df2176e9f56f0
#74735d43420cf1ed17fe0aa3494724808aefcf274572c7750be071a94560
#9c045c48a5454f9e7933bf93d967bcf582918b1bbd48f47dd978c7ea9c88
#23eb01482a40ef770f823ea2e63eaf7cab5367bd51b54979925464e27b22
#82fc5bfe94f6f41f2f91e56444ae371a050107fe79fd5db3f228bbbd5fcb
#653a87a3c1268d74500ba1489382290804d9d7937d3269b1c55eae51f9f7
#62eadbd4226ba6d4008414b353181ee6cfbe24204e4a82436b117c255014
#77ec74c3ac4d0203010001a38201003081fd301d0603551d0e041604149c
#0e12d3a222891fc1300b780f38b54e07e228073081cd0603551d230481c5
#3081c280149c0e12d3a222891fc1300b780f38b54e07e22807a1819ea481
#9b308198310b300906035504061302434131123010060355040813096361
#6c69666f6e69613111300f060355040713086d69706c69746173311b3019
#060355040a1312636973636f2073797374656d7320696e632e310e300c06
#0355040b13054e53535447311630140603550403130d436872697374696e
#652059616e311d301b06092a864886f70d010901160e6379616e40636973
#636f2e636f6d820900d48b1bbfb069e9b9300c0603551d13040530030101
#ff300d06092a864886f70d010105050003820101003e6d657e30c21acb30
#d7e91aef99cd6f83a8644d7ae437fb4a0cd1a8c6a0f1cb2194be80868b75
#1d20ee8dc72a1f80de697928820ec3f806cec67df136d6f8b6ff0ba7ef6f
#e4c228b37e7407becff9903198ebe2258d3774f198de8d0366cb10496408
#57dff62f247dfcae9813fde4167c1f959b291490cfb8e0b8db3395ce019f
#07c94aa4a86d1a0dae2b6cc6d139db740841dc65add58512ea59336dbc8c
#9abc14d500e42c8ea5031c201eec027d14e9878974629d2fdbe0d0cfffab
#8ab73f148bdec788de59550443ccf12cb3e74518274c3d7d2892eeb90f23
#0d5e853b85542c05b12198c097c2fb4023b370f0629645e11552f5429fb8
#aacef60a9344b5318202823082027e0201013081a6308198310b30090603
#55040613024341311230100603550408130963616c69666f6e6961311130
#0f060355040713086d69706c69746173311b3019060355040a1312636973
#636f2073797374656d7320696e632e310e300c060355040b13054e535354
#47311630140603550403130d436872697374696e652059616e311d301b06
#092a864886f70d010901160e6379616e40636973636f2e636f6d020900d4
#8b1bbfb069e9b9300906052b0e03021a0500a081b1301806092a864886f7
#0d010903310b06092a864886f70d010701301c06092a864886f70d010905
#310f170d3037313231383231323934375a302306092a864886f70d010904
#3116041431edc479779dcfbcfe8b821f0e8eb21dee0fae72305206092a86
#4886f70d01090f31453043300a06082a864886f70d0307300e06082a8648
#86f70d030202020080300d06082a864886f70d0302020140300706052b0e
#030207300d06082a864886f70d0302020128300d06092a864886f70d0101
#010500048201005d4705b9b6071da114d137fbefee02a23c52bd42127bc9
#a4f1574662bde953d7a1f5d74a9a264b4985336080b1fe51b82a24dc6ffb
#d9bdb1a43e64ba5807b9780ebf0a812df2be88d772846f6e4a642cfc3180
#cfc6ba852dd473851dfa5cbba3789b9f4535006799b281b97e75af45b5a5
#586d9c7f0c027b24c9c161cb5afc7abd0b027c36d82798f5a1bbabb2d2d6
#c7b12a8daa4942b5f2842de4b69bdb503c7a1f00cf1ce21f8aab5dca90a6
#d19127815cce00a13a553813e260ab4f1129b35af03793d67b278464ce95
#a1a5be8c6b0d8e7aff446d98b98fb441c5c95cb0350ded61e2a011e40b9a
#3f93f55160e7b0b9f4120143e17a3f7af7a5e4d76d3f3c
#
